# -*- coding: utf-8 -*-

# © Copyright 2011 Colin Powell. All Rights Reserved.
# This script is licensed under the BSD Open Source Licence
# Please see the text file LICENCE for more information
# If this script is distributed, it must be accompanied by the Licence

from datetime import datetime

from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _

from django_extensions.db.models import TimeStampedModel, TitleSlugDescriptionModel
from linkthru.managers import LinkThruManager

class LinkThruZone(TitleSlugDescriptionModel, TimeStampedModel):
    """
    a Model that describes the attributes and behaviours of a link zones

    A link zone is simply a location on a website so different size link thrus
    can be organized to display at different locations around the site.

    """
    class Meta:
        verbose_name = 'Link zone'
        verbose_name_plural = 'Link zones'

    def __unicode__(self):
        return "%s" % self.title

class LinkThruBase(TimeStampedModel):
    """
    This is our base model, from which all links will inherit.
    The manager methods for this model will determine which links to
    display return etc.

    """
    title = models.CharField(max_length=255)
    url = models.URLField(verify_exists=False)
    enabled = models.BooleanField(default=False)
    begins_on = models.DateTimeField(default=datetime.now)
    expires_on = models.DateTimeField(_('Expires on'), blank=True, null=True)
    updated = models.DateTimeField(editable=False, auto_now=True)
    zone = models.ForeignKey(LinkThruZone)

    objects = LinkThruManager()

    def __unicode__(self):
        return "%s" % self.title

    @models.permalink
    def get_view_url(self):
        return ('lt-linkthru-view', (), {'id': self.pk,})

    @models.permalink
    def get_absolute_url(self):
        return ('lt-linkthru-detail', (), {'id': self.pk,})

    def impressions(self, start=None, end=None):
        if start is not None:
            start_q=models.Q(impression_date__gte=start)
        else:
            start_q=models.Q()
        if end is not None:
            end_q=models.Q(impression_date__lte=end)
        else:
            end_q=models.Q()
        return self.adimpression_set.filter(start_q & end_q).count()

    def clicks(self, start=None, end=None):
        if start is not None:
            start_q=models.Q(click_date__gte=start)
        else:
            start_q=models.Q()
        if end is not None:
            end_q=models.Q(click_date__lte=end)
        else:
            end_q=models.Q()
        return self.adclick_set.filter(start_q & end_q).count()

class LinkImpression(models.Model):
    """
    The LinkImpression Model will record every time the ad is loaded on a page

    """
    impression_date = models.DateTimeField(auto_now=True)
    source_ip = models.IPAddressField(null=True, blank=True)
    link = models.ForeignKey(LinkThruBase)

    class Meta:
        verbose_name = 'Link Impression'
        verbose_name_plural = 'Link Impressions'

class LinkClick(models.Model):
    """
    The LinkClick model will record every click that a add gets

    """
    click_date = models.DateTimeField(default=datetime.now)
    source_ip = models.IPAddressField(null=True, blank=True)
    link = models.ForeignKey(LinkThruBase)

    class Meta:
        verbose_name = 'Link Click'
        verbose_name_plural = 'Link Clicks'

# Example Link Types
class TextLinkThru(LinkThruBase):
    """ A most basic, text based advert """
    content = models.TextField()

class ImageLinkThru(LinkThruBase):
    """ A standard image link thru """
    content = models.ImageField(upload_to="linkthru/images/")



# -*- coding: utf-8 -*-

# © Copyright 2009 Andre Engelbrecht. All Rights Reserved.
# This script is licensed under the BSD Open Source Licence
# Please see the text file LICENCE for more information
# If this script is distributed, it must be accompanied by the Licence

import logging
from django import template
from linkthru.models import LinkThruBase, LinkImpression
from datetime import datetime

register = template.Library()

@register.inclusion_tag('linkthru/link_tag.html', takes_context=True)
def random_zone_link(context, lin_zone):
    """
    Returns a random linkthru from the database.

    In order for the impression to be saved add the following
    to the TEMPLATE_CONTEXT_PROCESSORS:

    'linkthru.context_processors.get_source_ip'

    Tag usage:
    {% load linkthru_tags %}
    {% random_zone_link 'zone_slug' %}

    """
    to_return = {}

    # Retrieve a random link for the category and zone
    link = LinkThruBase.objects.get_random_link(link_zone)
    to_return['link'] = link
    
    # Record a impression for the link 
    if context.has_key('from_ip') and link:
        from_ip = context.get('from_ip')
        excluded_ip = context.get('excluded_ip')
        if not excluded_ip:
            try:
                impression = LinkImpression(
                        link=link,
                        impression_date=datetime.now(),
                        source_ip=from_ip
                )
                impression.save()
            except:
                pass
    return to_return

@register.inclusion_tag('linkthru/link_tag_block.html', takes_context=True)
def random_zone_link_block(context, link_zone, number):
    """
    Returns a block of random linkthrus from the database.

    In order for the impression to be saved add the following
    to the TEMPLATE_CONTEXT_PROCESSORS:

    'linkthru.context_processors.get_source_ip'

    Tag usage:
    {% load linkthru_tags %}
    {% random_zone_link_block 'zone_slug' 4 %}

    """
    to_return = {}

    # Retrieve a random link for the category and zone
    links=[]

    logging.debug('Looking up linkthrus for %s' % link_zone)
    # Check whether category has been specified
    lookup_links=LinkThruBase.objects.filter(enabled=True, zone__slug=link_zone)

    # If we have fewer links in system, adjust our block
    if len(lookup_links) < number: number = len(lookup_links)

    while(number > 0):
        new_link= LinkThruBase.objects.get_random_link(link_zone)
        if new_link in links:
            pass
        else:
            logging.debug('Adding ad %s to the list' % new_link)
            links.append(new_link)
            number=number-1
        
            # Record a impression for the link 
            if new_link:
                excluded_ip = context.get('excluded_ip')
                if not excluded_ip:
                    try:
                        impression = LinkImpression(
                                link=new_link,
                                impression_date=datetime.now(),
                                source_ip=context.get('from_ip')
                                )
                        impression.save()
                    except:
                        pass
    to_return['links'] = links
    return to_return




#! /usr/bin/env python
# emacs-mode: -*- python-*-
# -*- coding: utf-8 -*-
import logging
from datetime import datetime 
from django.shortcuts import render_to_response, get_object_or_404
from django.template.context import RequestContext
from django.http import HttpResponseRedirect 
from django.contrib.admin.views.decorators import staff_member_required
from linkthru.models import LinkThruBase, LinkClick, ImageLinkThru, LinkImpression

def linkthru_view(request, id):
    """
    Record the click in the database, then redirect to the thru url

    """
    link = get_object_or_404(LinkThruBase, id=id)
    logging.debug('Found linkthru: %s, redirecting...' % (link))
    try:
        click = LinkClick(link=link, click_date=datetime.now(), source_ip=request.META.get('REMOTE_ADDR'))
        click.save()
    except:
        raise Http404
    return HttpResponseRedirect(link.url)

def linkthru_index(request):
    links = LinkThruBase.objects.enabled()
    return render_to_response('linkthru/linkthru_list.html', locals(),
                              context_instance=RequestContext(request))

@staff_member_required
def linkthru_detail(request, id):
    link = get_object_or_404(LinkThruBase, id=id)
    return render_to_response('linkthru/linkthru_detail.html', locals(),
                              context_instance=RequestContext(request))






# -*- coding: utf-8 -*-

# © Copyright 2009 Andre Engelbrecht. All Rights Reserved.
# This script is licensed under the BSD Open Source Licence
# Please see the text file LICENCE for more information
# If this script is distributed, it must be accompanied by the Licence

from django.contrib import admin
from linkthru.models import LinkThruZone, ImageLinkThru, LinkClick, LinkImpression

class LinkThruZoneAdmin(admin.ModelAdmin):
    list_display = ['title', 'slug', 'description']


class ImageLinkThruAdmin(admin.ModelAdmin):
    fieldsets = [ (None,         {'fields': [('title', 'url'), ('enabled', 'begins_on'), 'zone', 'content']}), ]
    list_display = ['title', 'url', 'updated', 'enabled']
    list_filter = ['updated', 'enabled',]
    search_fields = ['title', 'url']

class LinkClickAdmin(admin.ModelAdmin):
    search_fields = ['link', 'source_ip']
    list_display = ['link', 'click_date', 'source_ip']
    list_filter = ['click_date']
    date_hierarchy = 'click_date'

class LinkImpressionAdmin(admin.ModelAdmin):
    search_fields = ['link', 'source_ip']
    list_display = ['link', 'impression_date', 'source_ip']
    list_filter = ['impression_date']
    date_hierarchy = 'impression_date'

admin.site.register(LinkThruZone, LinkThruZoneAdmin)
admin.site.register(ImageLinkThru, ImageLinkThruAdmin)
admin.site.register(LinkClick, LinkClickAdmin)
admin.site.register(LinkImpression, LinkImpressionAdmin)



from django.conf.urls.defaults import *

from linkthru.views import linkthru_view, linkthru_detail, linkthru_list

urlpatterns = patterns('',
    url(r'^view/(?P<id>[\d]+)/$', linkthru_view, name='lt-linkthru-view'),
    url(r'^featured/$', linkthru_list, name='lt-linkthru-list'),
    url(r'^linkthrus/(?P<id>[\d]+)/$', linkthru_detail, name='lt-linkthru-detail'),
    #url(r'^addisplay.html', ad_display, name='adzone_ad_display'),
    #url(r'^xhr_ads/(?P<format>\w+)$', xhr_ad_table, name='adzone_xhr_ad_table'),
)



from django.test import TestCase
from django.test.client import Client
from django.core.handlers.wsgi import WSGIRequest
from django.core.urlresolvers import reverse

from adzone.models import *
from django.contrib.auth.models import User

def datenow():
    from datetime import datetime
    return datetime.now()

class RequestFactory(Client):
    """
    Class that lets you create mock Request objects for use in testing.
    
    Usage:
    
    rf = RequestFactory()
    get_request = rf.get('/hello/')
    post_request = rf.post('/submit/', {'foo': 'bar'})
    
    This class re-uses the django.test.client.Client interface, docs here:
    http://www.djangoproject.com/documentation/testing/#the-test-client
    
    Once you have a request object you can pass it to any view function, 
    just as if that view had been hooked up using a URLconf.
    
    """
    def request(self, **request):
        """
        Similar to parent class, but returns the request object as soon as it
        has created it.
        """
        environ = {
            'HTTP_COOKIE': self.cookies,
            'PATH_INFO': '/',
            'QUERY_STRING': '',
            'REQUEST_METHOD': 'GET',
            'SCRIPT_NAME': '',
            'SERVER_NAME': 'testserver',
            'SERVER_PORT': 80,
            'SERVER_PROTOCOL': 'HTTP/1.1',
            'REMOTE_ADDR': '127.0.0.1',
        }
        environ.update(self.defaults)
        environ.update(request)
        return WSGIRequest(environ)

class AdvertisingTestCase(TestCase):
    def setUp(self):
        self.request = RequestFactory().request()

        self.user = User.objects.create_user('test', 'test@example.com', 'testpass')

        self.advertiser = Advertiser.objects.create(
                company_name = 'teh_node Web Development',
                website = 'http://andre.smoenux.webfactional.com/',
                user = self.user)

        # Categories setup
        self.category = AdCategory.objects.create(
                title = 'Internet Services',
                slug = 'internet-services',
                description = 'All internet based services')

        self.category2 = AdCategory.objects.create(
                title = 'Category Two',
                slug = 'categorytwo',
                description = 'A Second Category')

        # Zones setup
        self.adzone = AdZone.objects.create(
                title = 'Sidebar',
                slug = 'sidebar',
                description = 'Side Bar Ads')

        self.adzone2 = AdZone.objects.create(
                title = 'Content Banner',
                slug = 'contentbanner',
                description = 'Content Adverts')

        # Ad setup
        self.ad = TextAd.objects.create(
                title = 'First Ad',
                content = 'For all your web design and development needs, at competitive rates.',
                url = 'http://www.teh-node.co.za/',
                enabled = True,
                advertiser = self.advertiser,
                category = self.category,
                zone = self.adzone)

        self.ad2 = TextAd.objects.create(
                title = 'Second Ad',
                content = 'A second advert.',
                url = 'http://www.teh-node.co.za/',
                enabled = True,
                advertiser = self.advertiser,
                category = self.category2,
                zone = self.adzone2)

        self.ad3 = TextAd.objects.create(
                title = 'A Third Ad',
                content = 'A third advert.',
                url = 'http://www.teh-node.co.za/',
                enabled = True,
                advertiser = self.advertiser,
                category = self.category2,
                zone = self.adzone2)

        # AdImpression Setup
        self.impression1 = AdImpression.objects.create(
                impression_date=datenow(),
                source_ip='127.0.0.2',
                ad=self.ad)
        self.impression2 = AdImpression.objects.create(
                impression_date=datenow(),
                source_ip='127.0.0.3',
                ad=self.ad2)

        # Clicks Setup
        self.adclick1 = AdClick.objects.create(
                click_date=datenow(),
                source_ip='127.0.0.1',
                ad=self.ad)
        
class AdvertiserTestCase(AdvertisingTestCase):
    def testAdvertiser(self):
        self.assertEquals(self.advertiser.get_website_url(), 'http://andre.smoenux.webfactional.com/')

class CategoryTestCase(AdvertisingTestCase):
    def testAdCategory(self):
        self.assertEquals(self.category.__unicode__(), 'Internet Services')

class ZoneTestCase(AdvertisingTestCase):
    def testAdZone(self):
        self.assertEquals(self.adzone.__unicode__(), 'Sidebar')

    def testAdinZone(self):
        ads = TextAd.objects.filter(zone__slug='sidebar')
        self.assertEquals(len(ads), 1)

class AdvertTestCase(AdvertisingTestCase):
    def testAd(self):
        self.assertEquals(reverse(
            'adzone_ad_view',
            args=['1']
        )[-8:], '/view/1/')
        adimpressions = AdImpression.objects.filter(ad=self.ad)
        self.assertEquals(len(adimpressions), 1)
        self.assertEquals(adimpressions[0].source_ip, '127.0.0.2')

    def testAdAdvertiser(self):
        self.assertEquals(self.ad.advertiser.__unicode__(), 'teh_node Web Development')
        self.assertEquals(self.ad.advertiser.company_name, 'teh_node Web Development')

    def testAddsInCategory(self):
        ads = TextAd.objects.filter(category__slug='internet-services')
        self.assertEquals(len(ads), 1)
        self.assertEquals(ads[0].title, 'First Ad')

    def testRandomAd(self):
        ad = AdBase.objects.get_random_ad(
            ad_category='internet-services',
            ad_zone='sidebar'
        )
        self.assertEquals(ad.title, 'First Ad')

class ImpressionTestCase(AdvertisingTestCase):
    def testImpression(self):
        impressions = AdImpression.objects.all()
        self.assertEquals(len(impressions), 2)

class ClickTestCase(AdvertisingTestCase):
    def testClicks(self):
        clicks = AdClick.objects.all()
        self.assertEquals(len(clicks), 1)

    def testClickOnAds(self):
        c = Client(REMOTE_ADDR='127.0.0.1')
        response = c.get(reverse(
            'adzone_ad_view',
            args=['1']
        ))
        self.assertEquals(len(AdClick.objects.all()), 2)
        click = AdClick.objects.all()[1]
        self.assertEquals(click.source_ip, '127.0.0.1')

    def testInvalidAdURL(self):
        c = Client(REMOTE_ADDR='127.0.0.1')
        response = c.get('/te/10')
        self.assertEquals(len(AdClick.objects.all()), 1)
